#include "isp.h"

#include "common.h"
#include "hi_comm_vb.h"
#include "hi_comm_video.h"
#include "hi_common.h"
#include "hi_defines.h"
#include "hi_type.h"
#include "sample_comm.h"

#include <stdio.h>
#include <zlog.h>

#define VB_YUV_COUNT 10
#define VB_RGB_COUNT 4
#define ALIGN_DOWN_SIZE 2

zlog_category_t *log_isp = NULL;

// ********************
// * ISP Configuration*
// ********************
const PIXEL_FORMAT_E ISP_PIXEL_FORMAT = PIXEL_FORMAT_YVU_SEMIPLANAR_422;

const COMPRESS_MODE_E ISP_COMPRESS_MODE = COMPRESS_MODE_SEG;

// VI Configuration
SAMPLE_VI_CONFIG_S vi_config;
PIC_SIZE_E vi_pic_size_type = 0;

// VPSS Configuration
// Video VPSS
isp_vpss_config_t vpss_video_config = {0};
VPSS_CHN _vpss_video_chn[2] = {1, 2};
isp_vpss_chn_t vpss_video_chn = {_vpss_video_chn,
                                 ARRAY_LENGTH(_vpss_video_chn)};
// Snap VPSS
isp_vpss_config_t vpss_snap_config = {0};
VPSS_CHN _vpss_snap_chn[1] = {1};
isp_vpss_chn_t vpss_snap_chn = {_vpss_snap_chn, ARRAY_LENGTH(_vpss_snap_chn)};

// VENC Configurationk
VENC_CHN venc_video_chn = 0;
VENC_CHN venc_snap_chn = 1;

// VO Configuration
SAMPLE_VO_CONFIG_S vo_config = {0};

static HI_BOOL is_isp_working = HI_FALSE;

/**
 * @brief
 *
 * @param video_pipe
 * @param snap_pipe
 * @return HI_S32
 */
HI_S32 ispInit(VI_PIPE video_pipe, VI_PIPE snap_pipe) {
  HI_S32 ret = HI_SUCCESS;
  // VI Info
  SIZE_S pic_size = {0};
  HI_U32 frame_rate = 0;

  // VB Info
  VB_CONFIG_S vb_config = {0};

  // Init zlog
  log_isp = zlog_get_category("eb3516_isp");
  if (!log_isp) {
    printf("Failure: Failed to init zlog\n");
    return HI_FAILURE;
  }

  // ***************************
  // * Step1: VI Configuration *
  // ***************************
  // Get Sensor(VI) Info
  memset(&vi_config, 0, sizeof(SAMPLE_VI_CONFIG_S));
  SAMPLE_COMM_VI_GetSensorInfo(&vi_config);

  // Get frame rate
  ret = SAMPLE_COMM_VI_GetFrameRateBySensor(
      vi_config.astViInfo[0].stSnsInfo.enSnsType, &frame_rate);
  if (HI_SUCCESS != ret) {
    zlog_error(log_isp, "Failure: Get sensor frame rate failed with %d", ret);
    goto EXIT1;
  }

  // Input Device Info Configuration
  // Input Device Number
  vi_config.s32WorkingViNum = 1;
  vi_config.as32WorkingViId[0] = 0;
  // Sensor Info
  vi_config.astViInfo[0].stSnsInfo.MipiDev = SAMPLE_COMM_VI_GetComboDevBySensor(
      vi_config.astViInfo[0].stSnsInfo.enSnsType, 0);

  vi_config.astViInfo[0].stDevInfo.ViDev = 0;
  // Wide Dynamic Range Mode
  vi_config.astViInfo[0].stDevInfo.enWDRMode = WDR_MODE_NONE;
  // Input Pipe Info
  vi_config.astViInfo[0].stPipeInfo.aPipe[0] = video_pipe; // video pipe
  vi_config.astViInfo[0].stPipeInfo.aPipe[1] = snap_pipe;  // snap pipe
  vi_config.astViInfo[0].stPipeInfo.aPipe[2] = -1;
  vi_config.astViInfo[0].stPipeInfo.aPipe[3] = -1;
  vi_config.astViInfo[0].stPipeInfo.enMastPipeMode = VI_OFFLINE_VPSS_OFFLINE;
  // 设置 VI 通道信息
  vi_config.astViInfo[0].stChnInfo.ViChn = 0;
  vi_config.astViInfo[0].stChnInfo.enPixFormat =
      PIXEL_FORMAT_YVU_SEMIPLANAR_420;
  vi_config.astViInfo[0].stChnInfo.enDynamicRange = DYNAMIC_RANGE_SDR8;
  vi_config.astViInfo[0].stChnInfo.enVideoFormat = VIDEO_FORMAT_LINEAR;
  vi_config.astViInfo[0].stChnInfo.enCompressMode = ISP_COMPRESS_MODE;
  // 设置 VI 抓拍信息
  vi_config.astViInfo[0].stSnapInfo.bSnap = HI_TRUE;
  vi_config.astViInfo[0].stSnapInfo.bDoublePipe = HI_TRUE;
  vi_config.astViInfo[0].stSnapInfo.VideoPipe = video_pipe;
  vi_config.astViInfo[0].stSnapInfo.SnapPipe = snap_pipe;
  vi_config.astViInfo[0].stSnapInfo.enVideoPipeMode = VI_OFFLINE_VPSS_OFFLINE;
  vi_config.astViInfo[0].stSnapInfo.enSnapPipeMode = VI_OFFLINE_VPSS_OFFLINE;

  // ******************************************
  // * Step2: VB & MPI Configuration and Init *
  // ******************************************
  // Get sensor pic size type
  ret = SAMPLE_COMM_VI_GetSizeBySensor(
      vi_config.astViInfo[0].stSnsInfo.enSnsType, &vi_pic_size_type);
  if (HI_SUCCESS != ret) {
    zlog_error(log_isp, "Failure: Get sensor picture size failed with %d", ret);
    goto EXIT2;
  }
  // Convert pic type to struction
  ret = SAMPLE_COMM_SYS_GetPicSize(vi_pic_size_type, &pic_size);
  if (HI_SUCCESS != ret) {
    zlog_error(log_isp,
               "Failure: Convert pic type failed with wrong pic type %d",
               vi_pic_size_type);
    goto EXIT2;
  }

  memset(&vb_config, 0, sizeof(VB_CONFIG_S));
  // Set picture buffer
  vb_config.u32MaxPoolCnt = 2;
  // 获取一帧图片 buffer 的大小
  vb_config.astCommPool[0].u64BlkSize = COMMON_GetPicBufferSize(
      pic_size.u32Width, pic_size.u32Height, ISP_PIXEL_FORMAT, DATA_BITWIDTH_8,
      ISP_COMPRESS_MODE, DEFAULT_ALIGN);
  vb_config.astCommPool[0].u32BlkCnt = VB_YUV_COUNT;
  // 获取 raw buffer 的大小
  vb_config.astCommPool[1].u64BlkSize = VI_GetRawBufferSize(
      pic_size.u32Width, pic_size.u32Height, PIXEL_FORMAT_RGB_BAYER_16BPP,
      COMPRESS_MODE_NONE, DEFAULT_ALIGN);
  vb_config.astCommPool[1].u32BlkCnt = VB_RGB_COUNT;

  // 初始化 VB
  ret =
      SAMPLE_COMM_SYS_InitWithVbSupplement(&vb_config, VB_SUPPLEMENT_JPEG_MASK);
  if (ret != HI_SUCCESS) {
    zlog_error(log_isp, "Failure: Init VB and MPI failed with %#x!\n", ret);
    goto EXIT2;
  }

  // *******************
  // * Step3: VI  Init *
  // *******************
  ret = SAMPLE_COMM_VI_SetParam(&vi_config);
  if (HI_SUCCESS != ret) {
    zlog_error(log_isp, "Failure: Set VI params failed");
    goto EXIT3;
  }

  ISP_CTRL_PARAM_S stIspCtrlParam = {0};
  memset(&stIspCtrlParam, 0, sizeof(stIspCtrlParam));
  ret = HI_MPI_ISP_GetCtrlParam(vi_config.astViInfo[0].stPipeInfo.aPipe[0],
                                &stIspCtrlParam);
  stIspCtrlParam.u32StatIntvl = frame_rate / 30;
  ret = HI_MPI_ISP_SetCtrlParam(vi_config.astViInfo[0].stPipeInfo.aPipe[0],
                                &stIspCtrlParam);

  ret = SAMPLE_COMM_VI_StartVi(&vi_config);
  if (ret != HI_SUCCESS) {
    zlog_error(log_isp, "Failure: Start VI failed with %#x!\n", ret);
    goto EXIT3;
  }

  // Init Success
  is_isp_working = HI_TRUE;
  zlog_debug(log_isp, "Success: ISP Init Finish...");
  return HI_SUCCESS;

  // Failure Conditions
EXTI4:
  SAMPLE_COMM_VI_StopVi(&vi_config);
EXIT3:
  SAMPLE_COMM_SYS_Exit();
EXIT2:
EXIT1:

  return ret;
}

/**
 * @brief Stop ISP and Exit
 *
 */
void ispExit(void) {
  if (is_isp_working) {
    SAMPLE_COMM_VI_StopVi(&vi_config);
    SAMPLE_COMM_SYS_Exit();
    is_isp_working = HI_FALSE;
  }
  zlog_info(log_isp, "Success: ISP exit successfully");
}

HI_BOOL ispCheck(void) { return is_isp_working; }

SAMPLE_VI_CONFIG_S *ispGetViConfig(void) { return &vi_config; }

PIC_SIZE_E ispGetViPicSizeType(void) { return vi_pic_size_type; }

VI_PIPE ispGetVideoPipe(void) {
  return vi_config.astViInfo[0].stSnapInfo.VideoPipe;
}

VI_PIPE ispGetSnapPipe(void) {
  return vi_config.astViInfo[0].stSnapInfo.SnapPipe;
}

isp_vpss_config_t *ispGetVideoVpss(void) { return &vpss_video_config; }

isp_vpss_chn_t *ispGetVideoVpssChn(void) { return &vpss_video_chn; }

VENC_CHN ispGetVideoVencChn(void) { return venc_video_chn; }

isp_vpss_config_t *ispGetSnapVpss(void) { return &vpss_snap_config; }

isp_vpss_chn_t *ispGetSnapVpssChn(void) { return &vpss_snap_chn; }

VENC_CHN ispGetSnapVencChn(void) { return venc_snap_chn; }

SAMPLE_VO_CONFIG_S *ispGetVOConfig(void) { return &vo_config; }

/**
 * @brief Get Vpss Chn Attrbute
 *
 * @param vpss_chn_attr The struction of vpss chn attr
 * @param vpss_chn The number of vpss chn
 * @param pic_size The enum of picture size
 * @return HI_S32 success or failure
 */
HI_S32 ispGetVpssChnAttr(VPSS_CHN_ATTR_S *vpss_chn_attr, VPSS_CHN vpss_chn,
                         PIC_SIZE_E pic_size) {
  HI_S32 ret = HI_SUCCESS;

  // Convert pic size enum to struct
  SIZE_S size;
  ret = SAMPLE_COMM_SYS_GetPicSize(pic_size, &size);
  if (HI_SUCCESS != ret) {
    zlog_error(log_isp, "Failure: Failed to get picture size");
    return ret;
  }

  // Configuration
  vpss_chn_attr->enChnMode = VPSS_CHN_MODE_USER;
  vpss_chn_attr->u32Width = size.u32Width;
  vpss_chn_attr->u32Height = size.u32Height;
  vpss_chn_attr->enVideoFormat = VIDEO_FORMAT_LINEAR;
  vpss_chn_attr->enPixelFormat = PIXEL_FORMAT_YVU_SEMIPLANAR_420;
  vpss_chn_attr->enDynamicRange = DYNAMIC_RANGE_SDR8;
  vpss_chn_attr->enCompressMode = COMPRESS_MODE_NONE;
  vpss_chn_attr->stFrameRate.s32SrcFrameRate = -1;
  vpss_chn_attr->stFrameRate.s32DstFrameRate = -1;
  vpss_chn_attr->u32Depth = 1;
  vpss_chn_attr->bMirror = HI_FALSE;
  vpss_chn_attr->bFlip = HI_FALSE;
  vpss_chn_attr->stAspectRatio.enMode = ASPECT_RATIO_NONE;

  return ret;
}

/**
 * @brief Get Vpss configuration
 *
 * @param vpss_grp_id
 * @param vpss_config
 * @param vpss_chn
 * @param vpss_chn_num
 * @param pic_size
 * @param is_snap
 * @return HI_S32
 */
HI_S32 ispGetVpssConfig(VPSS_GRP vpss_grp_id, isp_vpss_config_t *vpss_config,
                        VPSS_CHN *vpss_chn, HI_U8 vpss_chn_num,
                        PIC_SIZE_E pic_size, HI_BOOL is_snap) {
  HI_S32 ret = HI_SUCCESS;

  // Assert vpss group id less than 16
  if (vpss_grp_id > VPSS_MAX_GRP_NUM) {
    zlog_error(log_isp, "Failure: VPSS group id should not over %d",
               VPSS_MAX_GRP_NUM);
    return HI_FAILURE;
  }
  zlog_debug(log_isp, "VPSS Group ID: %d", vpss_grp_id);

  // Assert vpss chn number less than 11
  if (vpss_chn_num > VPSS_MAX_CHN_NUM) {
    zlog_error(log_isp, "Failure: chn number should not over %d",
               VPSS_MAX_CHN_NUM);
    return HI_FAILURE;
  }

  // Convert pic size enum to struct
  SIZE_S size;
  ret = SAMPLE_COMM_SYS_GetPicSize(pic_size, &size);
  if (HI_SUCCESS != ret) {
    zlog_error(log_isp, "Failure: Failed to get picture size");
    return ret;
  }

  // Vpss Configuration
  vpss_config->s32GrpId = vpss_grp_id;
  vpss_config->stGrpAttr.u32MaxW = size.u32Width;
  vpss_config->stGrpAttr.u32MaxH = size.u32Width;
  vpss_config->stGrpAttr.enPixelFormat = PIXEL_FORMAT_YVU_SEMIPLANAR_420;
  vpss_config->stGrpAttr.enDynamicRange = DYNAMIC_RANGE_SDR8;
  vpss_config->stGrpAttr.stFrameRate.s32SrcFrameRate = -1;
  vpss_config->stGrpAttr.stFrameRate.s32DstFrameRate = -1;
  vpss_config->stGrpAttr.bNrEn = HI_TRUE;
  vpss_config->stGrpAttr.stNrAttr.enNrType =
      !is_snap ? VPSS_NR_TYPE_VIDEO : VPSS_NR_TYPE_SNAP;
  vpss_config->stGrpAttr.stNrAttr.enCompressMode = COMPRESS_MODE_FRAME;
  vpss_config->stGrpAttr.stNrAttr.enNrMotionMode = NR_MOTION_MODE_NORMAL;

  // Vpss Channel Configuration
  for (HI_S32 i = 0; i < vpss_chn_num; i++) {
    VPSS_CHN _vpss_chn = vpss_chn[i];
    if (_vpss_chn > VPSS_MAX_CHN_NUM) {
      zlog_error(
          log_isp,
          "Failure: Failed to set vpss chn: chn number should not over %d",
          VPSS_MAX_CHN_NUM);
      return HI_FAILURE;
    }

    // Enable Chn and set Chn Attr
    vpss_config->abChnEnable[_vpss_chn] = HI_TRUE;
    ret = ispGetVpssChnAttr(&vpss_config->astChnAttrs[_vpss_chn], _vpss_chn,
                            PIC_1080P);
    if (HI_SUCCESS != ret) {
      zlog_error(log_isp, "Failure: Failed to set vpss chn");
      return HI_FAILURE;
    }
  }

  return ret;
}

void ispPrintVpssErrorCode(void) {
  zlog_info(log_isp, "HI_ERR_VPSS_NULL_PTR         : 0x%x",
            HI_ERR_VPSS_NULL_PTR);
  zlog_info(log_isp, "HI_ERR_VPSS_NOTREADY         : 0x%x",
            HI_ERR_VPSS_NOTREADY);
  zlog_info(log_isp, "HI_ERR_VPSS_INVALID_DEVID    : 0x%x",
            HI_ERR_VPSS_INVALID_DEVID);
  zlog_info(log_isp, "HI_ERR_VPSS_INVALID_CHNID    : 0x%x",
            HI_ERR_VPSS_INVALID_CHNID);
  zlog_info(log_isp, "HI_ERR_VPSS_EXIST            : 0x%x", HI_ERR_VPSS_EXIST);
  zlog_info(log_isp, "HI_ERR_VPSS_UNEXIST          : 0x%x",
            HI_ERR_VPSS_UNEXIST);
  zlog_info(log_isp, "HI_ERR_VPSS_NOT_SUPPORT      : 0x%x",
            HI_ERR_VPSS_NOT_SUPPORT);
  zlog_info(log_isp, "HI_ERR_VPSS_NOT_PERM         : 0x%x",
            HI_ERR_VPSS_NOT_PERM);
  zlog_info(log_isp, "HI_ERR_VPSS_NOMEM            : 0x%x", HI_ERR_VPSS_NOMEM);
  zlog_info(log_isp, "HI_ERR_VPSS_NOBUF            : 0x%x", HI_ERR_VPSS_NOBUF);
  zlog_info(log_isp, "HI_ERR_VPSS_SIZE_NOT_ENOUGH  : 0x%x",
            HI_ERR_VPSS_SIZE_NOT_ENOUGH);
  zlog_info(log_isp, "HI_ERR_VPSS_ILLEGAL_PARAM    : 0x%x",
            HI_ERR_VPSS_ILLEGAL_PARAM);
  zlog_info(log_isp, "HI_ERR_VPSS_BUSY             : 0x%x", HI_ERR_VPSS_BUSY);
  zlog_info(log_isp, "HI_ERR_VPSS_BUF_EMPTY        : 0x%x",
            HI_ERR_VPSS_BUF_EMPTY);
}