#include "snap.h"
#include "hi_type.h"
#include "isp.h"

#include "hi_comm_snap.h"
#include "hi_comm_video.h"
#include "hi_common.h"
#include "mpi_snap.h"
#include "sample_comm.h"

#include <zlog.h>

extern zlog_category_t *log_isp;

static HI_BOOL is_isp_snap_working = HI_FALSE;

HI_S32 ispStartSnap(void) {
  HI_S32 ret = HI_SUCCESS;

  // VI Info
  SAMPLE_VI_CONFIG_S *vi_config = ispGetViConfig();
  VI_PIPE snap_pipe = ispGetSnapPipe();
  VI_CHN vi_chn = vi_config->astViInfo[0].stChnInfo.ViChn;
  PIC_SIZE_E vi_pic_size_type = ispGetViPicSizeType();

  // VPSS Info
  isp_vpss_config_t *vpss_snap_config = ispGetSnapVpss();
  isp_vpss_chn_t *vpss_snap_chn = ispGetSnapVpssChn();

  // Venc Info
  VENC_CHN venc_snap_chn = ispGetSnapVencChn();

#ifdef DEBUG
  ispPrintVpssErrorCode();
#endif // DEBUG

  // *******************************
  // * Step1: VPSS Config and Init *
  // *******************************
  // Get VPSS config
  ret = ispGetVpssConfig(
      (VPSS_GRP)1, vpss_snap_config, vpss_snap_chn->as32VpssChn,
      vpss_snap_chn->u8VpssChnNum, vi_pic_size_type, HI_TRUE);
  if (HI_SUCCESS != ret) {
    zlog_error(log_isp, "Failure: Failed to get Snap VPSS config");
    goto EXIT1;
  }
  // Start VPSS
  ret = SAMPLE_COMM_VPSS_Start(
      vpss_snap_config->s32GrpId, vpss_snap_config->abChnEnable,
      &vpss_snap_config->stGrpAttr, vpss_snap_config->astChnAttrs);
  if (ret != HI_SUCCESS) {
    zlog_error(log_isp, "Failure: Start Snap VPSS group %d failed with %#x!\n",
               vpss_snap_config->s32GrpId, ret);
    goto EXIT1;
  }

  // **************************
  // * Step2: Bind VI -> VPSS *
  // **************************
  ret = SAMPLE_COMM_VI_Bind_VPSS(snap_pipe, vi_chn, vpss_snap_config->s32GrpId);
  if (ret != HI_SUCCESS) {
    zlog_error(log_isp,
               "Failure: VI Video Pipe bind Snap VPSS failed with %#x!\n", ret);
    goto EXIT2;
  }

  // *************************
  // * Step3: Star Snap Venc *
  // *************************
  SIZE_S vi_pic_size;
  ret = SAMPLE_COMM_SYS_GetPicSize(vi_pic_size_type, &vi_pic_size);
  if (HI_SUCCESS != ret) {
    zlog_error(log_isp,
               "Failure: Convert pic type failed with wrong pic type %d",
               vi_pic_size_type);
    goto EXIT2;
  }
  zlog_debug(log_isp, "Debug: VI pic size type = %d", vi_pic_size_type);
  // Start snap venc
  ret = SAMPLE_COMM_VENC_SnapStart(venc_snap_chn, &vi_pic_size, HI_TRUE);
  if (HI_SUCCESS != ret) {
    zlog_error(log_isp, "Failure: Venc Snap Start failed with %#x\n", ret);
    goto EXIT2;
  }

  // *********************************
  // * Step4: Bind VPSS -> Snap Venc *
  // *********************************
  ret = SAMPLE_COMM_VPSS_Bind_VENC(
      vpss_snap_config->s32GrpId, vpss_snap_chn->as32VpssChn[0], venc_snap_chn);
  if (HI_SUCCESS != ret) {
    zlog_error(log_isp, "Failure: Venc bind Vpss failed with %#x!\n", ret);
    goto EXIT3;
  }

  // ***************************
  // * Step4: Enable Snap Pipe *
  // ***************************
  SNAP_ATTR_S stSnapAttr;
  stSnapAttr.enSnapType = SNAP_TYPE_NORMAL;
  stSnapAttr.bLoadCCM = HI_TRUE;
  stSnapAttr.stNormalAttr.u32FrameCnt = 1;
  stSnapAttr.stNormalAttr.u32RepeatSendTimes = 1;
  stSnapAttr.stNormalAttr.bZSL = HI_FALSE;
  ret = HI_MPI_SNAP_SetPipeAttr(snap_pipe, &stSnapAttr);
  if (HI_SUCCESS != ret) {
    zlog_error(log_isp, "Failure: HI_MPI_SNAP_SetPipeAttr failed with %#x!\n",
               ret);
    goto EXIT4;
  }
  ret = HI_MPI_SNAP_EnablePipe(snap_pipe);
  if (HI_SUCCESS != ret) {
    zlog_error(log_isp, "Failure: HI_MPI_SNAP_EnablePipe failed with %#x!\n",
               ret);
    goto EXIT4;
  }

  // // *************
  // // * Test Snap *
  // // *************
  // printf("=======press any key to trigger=====\n");
  // getchar();

  // ret = HI_MPI_SNAP_TriggerPipe(snap_pipe);
  // if (HI_SUCCESS != ret) {
  //   SAMPLE_PRT("HI_MPI_SNAP_TriggerPipe failed with %#x!\n", ret);
  // }

  // ret = SAMPLE_COMM_VENC_SnapProcess(
  //     venc_snap_chn, stSnapAttr.stNormalAttr.u32FrameCnt, HI_TRUE, HI_TRUE);
  // if (HI_SUCCESS != ret) {
  //   printf("%s: sanp process failed!\n", __FUNCTION__);
  // }

  // printf("snap success!\n");

  is_isp_snap_working = HI_TRUE;

  return ret;

// Failure Conditions
EXIT5:
  HI_MPI_SNAP_DisablePipe(snap_pipe);
EXIT4:
  SAMPLE_COMM_VPSS_UnBind_VENC(vpss_snap_config->s32GrpId,
                               vpss_snap_chn->as32VpssChn[0], venc_snap_chn);
EXIT3:
  SAMPLE_COMM_VENC_SnapStop(venc_snap_chn);
EXIT2:
  SAMPLE_COMM_VPSS_Stop(vpss_snap_config->s32GrpId,
                        vpss_snap_config->abChnEnable);
EXIT1:
  ispExit();

  return ret;
}

void ispStopSnap(void) {
  // VI Info
  VI_PIPE snap_pipe = ispGetSnapPipe();

  // VPSS Info
  isp_vpss_config_t *vpss_snap_config = ispGetSnapVpss();
  isp_vpss_chn_t *vpss_snap_chn = ispGetSnapVpssChn();

  // Venc Info
  VENC_CHN venc_snap_chn = ispGetSnapVencChn();

  HI_MPI_SNAP_DisablePipe(snap_pipe);
  SAMPLE_COMM_VPSS_UnBind_VENC(vpss_snap_config->s32GrpId,
                               vpss_snap_chn->as32VpssChn[0], venc_snap_chn);
  SAMPLE_COMM_VENC_SnapStop(venc_snap_chn);
  SAMPLE_COMM_VPSS_Stop(vpss_snap_config->s32GrpId,
                        vpss_snap_config->abChnEnable);
  ispExit();
  is_isp_snap_working = HI_FALSE;
}

HI_BOOL ispCheckSnap(void) { return is_isp_snap_working; }

HI_S32 ispSnapInstantly(void) {
  HI_S32 ret = HI_SUCCESS;

  // VI Info
  VI_PIPE snap_pipe = ispGetSnapPipe();

  // Venc Info
  VENC_CHN venc_snap_chn = ispGetSnapVencChn();

  if (!is_isp_snap_working) {
    zlog_warn(log_isp, "Snap Failed, Because ISP Snap Module Not Work");
    return HI_FAILURE;
  }

  zlog_info(log_isp, "Ready to snap");

  ret = HI_MPI_SNAP_TriggerPipe(snap_pipe);
  if (HI_SUCCESS != ret) {
    zlog_error(log_isp, "Failure: HI_MPI_SNAP_TriggerPipe failed with %#x!\n",
               ret);
    return HI_FAILURE;
  }

  ret = SAMPLE_COMM_VENC_SnapProcess(venc_snap_chn, 1, HI_TRUE, HI_TRUE);
  if (HI_SUCCESS != ret) {
    zlog_error(log_isp, "Failure: %s: sanp process failed!\n", __FUNCTION__);
    return HI_FAILURE;
  }

  zlog_info(log_isp, "Success: Snap successfully");
  return HI_SUCCESS;
}

HI_S32 ispSnapByPressKey(void) {
  HI_S32 ret = HI_SUCCESS;

  // VI Info
  VI_PIPE snap_pipe = ispGetSnapPipe();

  // Venc Info
  VENC_CHN venc_snap_chn = ispGetSnapVencChn();

  if (!is_isp_snap_working) {
    zlog_warn(log_isp, "Snap Failed, Because ISP Snap Module Not Work");
    return HI_FAILURE;
  }

  zlog_info(log_isp, "Ready to snap");
  zlog_info(log_isp, "=======press any key to trigger=======");
  getchar();

  ret = HI_MPI_SNAP_TriggerPipe(snap_pipe);
  if (HI_SUCCESS != ret) {
    zlog_error(log_isp, "Failure: HI_MPI_SNAP_TriggerPipe failed with %#x!\n",
               ret);
    return HI_FAILURE;
  }

  ret = SAMPLE_COMM_VENC_SnapProcess(venc_snap_chn, 1, HI_TRUE, HI_TRUE);
  if (HI_SUCCESS != ret) {
    zlog_error(log_isp, "Failure: %s: sanp process failed!\n", __FUNCTION__);
    return HI_FAILURE;
  }

  zlog_info(log_isp, "Success: Snap successfully");

  return HI_SUCCESS;
}
