#include "video.h"
#include "isp.h"

#include "hi_common.h"
#include "hi_type.h"
#include "sample_comm.h"
#include <zlog.h>

extern zlog_category_t *log_isp;

static HI_BOOL is_isp_video_working = HI_FALSE;

HI_S32 ispStartVideo(void) {
  HI_S32 ret = HI_SUCCESS;

  // VI Info
  SAMPLE_VI_CONFIG_S *vi_config = ispGetViConfig();
  VI_PIPE video_pipe = ispGetVideoPipe();
  VI_CHN vi_chn = vi_config->astViInfo[0].stChnInfo.ViChn;
  PIC_SIZE_E vi_pic_size_type = ispGetViPicSizeType();

  // VPSS Info
  isp_vpss_config_t *vpss_video_config = ispGetVideoVpss();
  isp_vpss_chn_t *vpss_video_chn = ispGetVideoVpssChn();

  // VO Info
  SAMPLE_VO_CONFIG_S *vo_config = ispGetVOConfig();

  // Check ISP working status
  if (!ispCheck()) {
    zlog_error(log_isp, "Failure: ISP not working!");
    return HI_FAILURE;
  }

#ifdef DEBUG
  ispPrintVpssErrorCode();
#endif // DEBUG

  // *******************************
  // * Step1: VPSS Config and Init *
  // *******************************
  // Get VPSS config
  ret = ispGetVpssConfig(
      (VPSS_GRP)0, vpss_video_config, vpss_video_chn->as32VpssChn,
      vpss_video_chn->u8VpssChnNum, vi_pic_size_type, HI_FALSE);
  // SIZE_S stSize;
  // SAMPLE_COMM_SYS_GetPicSize(vi_pic_size_type, &stSize);
  // ret = VPSS_GetParams((SAMPLE_VPSS_CONFIG_S *)vpss_video_config, &stSize,
  //                      HI_FALSE);
  if (HI_SUCCESS != ret) {
    zlog_error(log_isp, "Failure: Failed to get Video VPSS config");
    goto EXIT1;
  }
  // Start VPSS
  ret = SAMPLE_COMM_VPSS_Start(
      vpss_video_config->s32GrpId, vpss_video_config->abChnEnable,
      &vpss_video_config->stGrpAttr, vpss_video_config->astChnAttrs);
  if (ret != HI_SUCCESS) {
    zlog_error(log_isp, "Failure: Start Video VPSS group %d failed with %#x!\n",
               vpss_video_config->s32GrpId, ret);
    goto EXIT1;
  }

  // **************************
  // * Step2: Bind VI -> VPSS *
  // **************************
  ret =
      SAMPLE_COMM_VI_Bind_VPSS(video_pipe, vi_chn, vpss_video_config->s32GrpId);
  if (ret != HI_SUCCESS) {
    zlog_error(log_isp, "Failure: VI Video Pipe bind Video VPSS failed with %#x!\n",
               ret);
    goto EXIT2;
  }

  // *******************************
  // * Step3: VO Config and Init *
  // *******************************
  // VO config
  ret = SAMPLE_COMM_VO_GetDefConfig(vo_config);
  if (HI_SUCCESS != ret) {
    zlog_error(log_isp, "Failure: Get VO config failed");
    goto EXIT3;
  }
  vo_config->enDstDynamicRange = DYNAMIC_RANGE_SDR8;
  vo_config->enVoIntfType = VO_INTF_HDMI;
  vo_config->enPicSize = vi_pic_size_type;

  // Start Vo
  ret = SAMPLE_COMM_VO_StartVO(vo_config);
  if (HI_SUCCESS != ret) {
    zlog_error(log_isp, "Failure: Start vo failed. s32Ret: 0x%x !", ret);
    goto EXIT3;
  }

  // ********************************
  // * Step4: Bind VPSS -> VO(HDMI) *
  // ********************************
  ret = SAMPLE_COMM_VPSS_Bind_VO(vpss_video_config->s32GrpId,
                                 vpss_video_chn->as32VpssChn[0],
                                 vo_config->VoDev, (VO_CHN)0);
  if (HI_SUCCESS != ret) {
    zlog_error(log_isp, "Failure: VO bind vpss failed. s32Ret: 0x%x !\n", ret);
    goto EXIT4;
  }

  is_isp_video_working = HI_TRUE;
  zlog_debug(log_isp, "Success: ISP Video Module Start Successfully!");
  return ret;

// Failure Conditions
EXIT5:
  SAMPLE_COMM_VPSS_UnBind_VO(vpss_video_config->s32GrpId,
                             vpss_video_chn->as32VpssChn[0], vo_config->VoDev,
                             (VO_CHN)0);
EXIT4:
  SAMPLE_COMM_VO_StopVO(vo_config);
EXIT3:
  SAMPLE_COMM_VI_UnBind_VPSS(video_pipe, vi_chn, vpss_video_config->s32GrpId);
EXIT2:
  SAMPLE_COMM_VPSS_Stop(vpss_video_config->s32GrpId,
                        vpss_video_config->abChnEnable);
EXIT1:
  ispExit();

  return ret;
}

void ispStopVideo(void) {
  // VI Info
  SAMPLE_VI_CONFIG_S *vi_config = ispGetViConfig();
  VI_PIPE video_pipe = ispGetVideoPipe();
  VI_CHN vi_chn = vi_config->astViInfo[0].stChnInfo.ViChn;

  // VPSS Info
  isp_vpss_config_t *vpss_video_config = ispGetVideoVpss();
  isp_vpss_chn_t *vpss_video_chn = ispGetVideoVpssChn();

  // VO Info
  SAMPLE_VO_CONFIG_S *vo_config = ispGetVOConfig();

  // Stop VPSS and VO
  SAMPLE_COMM_VPSS_UnBind_VO(vpss_video_config->s32GrpId,
                             vpss_video_chn->as32VpssChn[0], vo_config->VoDev,
                             (VO_CHN)0);
  SAMPLE_COMM_VO_StopVO(vo_config);
  SAMPLE_COMM_VI_UnBind_VPSS(video_pipe, vi_chn, vpss_video_config->s32GrpId);
  SAMPLE_COMM_VPSS_Stop(vpss_video_config->s32GrpId,
                        vpss_video_config->abChnEnable);
  // Stop isp
  if (ispCheck()) {
    ispExit();
  }

  is_isp_video_working = HI_FALSE;
}

HI_BOOL ispCheckVideo(void) { return is_isp_video_working; }